import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.util.List;
import java.util.ArrayList;

public class AcgDAO extends GenericDAO<Acg, Integer> {

   public AcgDAO() {
      sqlInsert = "INSERT INTO acg (descr, ch) VALUES (?, ?)";
      sqlSelect = "SELECT id, descr, ch FROM acg WHERE id = ?";
      sqlUpdate = "UPDATE acg SET descr = ?, ch = ? WHERE id = ?";
      sqlDelete = "DELETE FROM acg WHERE id = ?";
      sqlSelectAll = "SELECT * FROM acg";
      sqlCreate = "CREATE TABLE acg " +
         "(id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) PRIMARY KEY," +
         " descr VARCHAR(255), ch INTEGER)";
      sqlDrop = "DROP TABLE acg IF EXISTS";
   }

   public AcgDAO(Connection conn) {
      super(conn);
   }

   protected void fillSqlInsert(PreparedStatement ps, Acg a) throws SQLException {
      //ps.setInt(1, a.getId());
      ps.setString(1, a.getDescr());
      ps.setInt(2, a.getCh());
   }

   protected void fillSqlSelect(PreparedStatement ps, Integer id) throws SQLException {
      ps.setInt(1, id);
   }

   protected void fillSqlUpdate(PreparedStatement ps, Acg a) throws SQLException {
      ps.setString(1, a.getDescr());
      ps.setInt(2, a.getCh());
      ps.setInt(3, a.getId());
   }

   protected void fillSqlDelete(PreparedStatement ps, Integer id) throws SQLException {
      ps.setInt(1, id);
   }

   protected Acg fillFrom(ResultSet rs) throws SQLException {
      Acg a = new Acg();
      a.setId(rs.getInt("id"));
      a.setDescr(rs.getString("descr"));
      a.setCh(rs.getInt("ch"));
      return a;
   }

}
